from django.contrib.auth.models import User
from django.test import TestCase, Client
from django.urls import reverse


def login_user(c: object, self: object) -> object:
    response = c.post(
        reverse('login'),
        self.credentials,
        follow=True,
    )
    self.assertTrue(response.context['user'].is_active)


class AdminUserPermissionTest(TestCase):
    """
    The admin user will have full access to the whole site - even if they are not associated with
    a group that is associated with the object.
    """
    fixtures = ['NearBeach_basic_setup.json']

    def setUp(self):
        self.credentials = {
            'username': 'admin',
            'password': 'Test1234$'
        }

    def test_project_permissions(self):
        c = Client()

        # User will be logged in
        login_user(c, self)


        # Make sure the admin user can open up the project
        response = c.get(reverse('project_information', args=['1']))
        self.assertEqual(response.status_code, 200)
        print("Admin user can access a project with overlapping groups")

        # Make sure the admin user can open up the project
        response = c.get(reverse('project_information', args=['2']))
        self.assertEqual(response.status_code, 200)
        print("Admin user can access a project without overlapping groups")


class TeamLeaderPermissionTest(TestCase):
    """
    The team leader will only have access to objects that have at least one cross over group with that
    particular team leader.
    """
    fixtures = ['NearBeach_basic_setup.json']

    def setUp(self):
        self.credentials = {
            'username': 'team_leader',
            'password': 'Test1234$'
        }

    def test_project_permissions(self):
        c = Client()

        # User will be logged in
        login_user(c, self)

        # Make sure the admin user can open up the project
        response = c.get(reverse('project_information', args=['1']))
        self.assertEqual(response.status_code, 200)
        print("Team Leader can access a project with overlapping groups")

        # # Make sure the admin user can open up the project
        # response = c.get(reverse('project_information', args=['2']))
        # self.assertEqual(response.status_code, 403)
        # print("Team Lead can NOT access a project without overlapping groups")


class TeamMemberPermissionTest(TestCase):
    """
    The team MEMBER will only have access to objects that have at least one cross over group with that
    particular team leader.
    """
    fixtures = ['NearBeach_basic_setup.json']

    def setUp(self):
        self.credentials = {
            'username': 'team_member',
            'password': 'Test1234$'
        }

    def test_project_permissions(self):
        c = Client()

        # User will be logged in
        login_user(c, self)

        # Make sure the admin user can open up the project
        response = c.get(reverse('project_information', args=['1']))
        self.assertEqual(response.status_code, 200)
        print("Team Member can access a project with overlapping groups")

        # # Make sure the admin user can open up the project
        # response = c.get(reverse('project_information', args=['2']))
        # self.assertEqual(response.status_code, 403)
        # print("Team Member can NOT access a project without overlapping groups")


class TeamInternPermissionTest(TestCase):
    """
    The team leader will only have access to objects that have at least one cross over group with that
    particular team leader.
    """
    fixtures = ['NearBeach_basic_setup.json']

    def setUp(self):
        self.credentials = {
            'username': 'team_intern',
            'password': 'Test1234$'
        }

    def test_project_permissions(self):
        c = Client()

        # User will be logged in
        login_user(c, self)

        # Make sure the admin user can open up the project
        response = c.get(reverse('project_information', args=['1']))
        self.assertEqual(response.status_code, 200)
        print("Team Intern can access a project with overlapping groups")

        # # Make sure the admin user can open up the project
        # response = c.get(reverse('project_information', args=['2']))
        # self.assertEqual(response.status_code, 403)
        # print("Team Intern can NOT access a project without overlapping groups")


class ReadOnlyPermissionTest(TestCase):
    """
    The read only user should be forwarded to the read only template, if they have access to at least
    one group that the object has been assigned to.
    """
    fixtures = ['NearBeach_basic_setup.json']

    def setUp(self):
        self.credentials = {
            'username': 'read_only',
            'password': 'Test1234$'
        }

    def test_project_permissions(self):
        c = Client()

        # User will be logged in
        login_user(c, self)

        # Make sure the admin user can open up the project
        response = c.get(reverse('project_information', args=['1']))
        self.assertEqual(response.status_code, 200)
        print("Read only can access a project with overlapping groups")

        # Make sure the admin user can open up the project
        # response = c.get(reverse('project_information', args=['2']))
        # self.assertEqual(response.status_code, 403)
        # print("Read Only can NOT access a project without overlapping groups")



# Password reset
from django.contrib.auth import views as auth_views
from django.urls import path

from .views import admin_views, \
    authentication_views, \
    change_task_views, \
    customer_views, \
    dashboard_views, \
    document_views, \
    error_views, \
    group_views, \
    kanban_views, \
    object_data_views, \
    organisation_views, \
    permission_set_views, \
    project_views, \
    request_for_change_views, \
    requirement_item_views, \
    requirement_views, \
    search_views, \
    task_views, \
    user_views

# The following two imports are for the static files
# 404 and 500 pages

urlpatterns = [
    path('', dashboard_views.dashboard, name='dashboard'),

    # Administration
    path('admin/add_user/', admin_views.add_user, name='admin_add_user'),

    # Change Task
    path('change_task_update_status/<int:change_task_id>/', change_task_views.update_status,
         name='change_task_update_status'),

    # Customer
    path('customer_information/<int:customer_id>/', customer_views.customer_information, name='customer_information'),
    path('customer_information/<int:customer_id>/save/', customer_views.customer_information_save,
         name='customer_information_save'),

    # Dashboard
    path('dashboard/get/bug_list/', dashboard_views.get_bug_list, name='get_bug_list'),
    path('dashboard/get/my_objects/', dashboard_views.get_my_objects, name='get_my_objects'),
    path('dashboard/get/rfc_approvals/', dashboard_views.rfc_approvals, name='rfc_approvals'),

    # Documentation
    path('documentation/<destination>/<location_id>/add_folder/', document_views.document_add_folder,
         name='document_add_folder'),
    path('documentation/<destination>/<location_id>/add_link/', document_views.document_add_link,
         name='document_add_link'),
    path('documentation/<destination>/<location_id>/list/files/', document_views.document_list_files,
         name='document_list_files'),
    path('documentation/<destination>/<location_id>/list/folders/', document_views.document_list_folders,
         name='document_list_folders'),
    path('documentation/<destination>/<location_id>/upload/', document_views.document_upload, name='document_upload'),
    path('documentation/get/max_upload/', document_views.get_max_upload, name='document_get_max_upload'),

    # Groups
    path('group_information/<int:group_id>/', group_views.group_information, name='group_information'),
    path('group_information/<int:group_id>/save/', group_views.group_information_save, name='group_information_save'),
    path('group_information/check_group_name/', group_views.check_group_name, name='check_group_name'),

    # Kanban
    path('kanban_information/<int:kanban_board_id>/', kanban_views.kanban_information, name='kanban_information'),
    path('kanban_information/<int:kanban_board_id>/<object_lookup>/add_link/', kanban_views.add_kanban_link,
         name='add_kanban_link'),
    path('kanban_information/<int:kanban_board_id>/<object_lookup>/link_list/', kanban_views.kanban_link_list,
         name='kanban_link_list'),
    path('kanban_information/<int:kanban_board_id>/new_card/', kanban_views.new_kanban_card, name='new_kanban_card'),
    path('kanban_information/<int:kanban_card_id>/move_card/', kanban_views.move_kanban_card, name='move_kanban_card'),
    path('kanban_information/check_kanban_board_name/', kanban_views.check_kanban_board_name,
         name='check_kanban_board_name'),
    path('kanban_information/update_card/', kanban_views.update_card, name='kanban_update_card'),

    # Authentication
    path('login', authentication_views.login, name='login'),
    path('logout', authentication_views.logout, name='logout'),

    # Permission Sets
    path('permission_set_information/<int:permission_set_id>/', permission_set_views.permission_set_information,
         name='permission_set_information'
    ),
    path('permission_set_information/<int:permission_set_id>/save/',
         permission_set_views.permission_set_information_save,
         name='permission_set_information_save'
    ),

    # Private files
    path('private/<uuid:document_key>/', document_views.private_download_file, name='private_download_file'),

    # New Objects
    path('new_customer/', customer_views.new_customer, name='new_customer'),
    path('new_customer/save/', customer_views.new_customer_save, name='new_customer_save'),
    path('new_group/', group_views.new_group, name='new_group'),
    path('new_group/save/', group_views.new_group_save, name='new_group_save'),
    path('new_kanban/', kanban_views.new_kanban, name='new_kanban'),
    path('new_kanban_save/', kanban_views.new_kanban_save, name='new_kanban_save'),
    path('new_organisation/', organisation_views.new_organisation, name='new_organisation'),
    path('new_organisation/save/', organisation_views.new_organisation_save, name='new_organisation_save'),
    path('new_permission_set/', permission_set_views.new_permission_set, name='new_permission_set'),
    path('new_permission_set/save/', permission_set_views.new_permission_set_save, name='new_permission_set_save'),
    path('new_project/', project_views.new_project, name='new_project'),
    path('new_project/save/', project_views.new_project_save, name='new_project_save'),
    path('new_request_for_change/', request_for_change_views.new_request_for_change, name='new_request_for_change'),
    path('new_request_for_change/save/', request_for_change_views.new_request_for_change_save,
         name='new_request_for_change_save'),
    path('new_requirement/', requirement_views.new_requirement, name='new_requirement'),
    path('new_requirement/save/', requirement_views.new_requirement_save, name='new_requirement_save'),
    path('new_requirement_item/save/<int:requirement_id>/', requirement_item_views.new_requirement_item,
         name='new_requirement_item'),
    path('new_task/', task_views.new_task, name='new_task'),
    path('new_task/save/', task_views.new_task_save, name='new_task_save'),
    path('new_user/', user_views.new_user, name='new_user'),
    path('new_user/save/', user_views.new_user_save, name='new_user_save'),

    # Object Data
    path('object_data/admin_add_user/', object_data_views.admin_add_user, name='admin_add_user'),
    path('object_data/<destination>/<location_id>/add_bug/', object_data_views.add_bug, name='add_bug'),
    path('object_data/<destination>/<location_id>/add_customer/', object_data_views.add_customer, name='add_customer'),
    path('object_data/<destination>/<location_id>/add_group/', object_data_views.add_group, name='add_group'),
    path('object_data/<destination>/<location_id>/add_link/', object_data_views.add_link, name='add_link'),
    path('object_data/<destination>/<location_id>/add_notes/', object_data_views.add_notes, name='add_notes'),
    path('object_data/<destination>/<location_id>/add_user/', object_data_views.add_user, name='add_user'),
    path('object_data/<destination>/<location_id>/associated_objects/', object_data_views.associated_objects,
         name='associated_objects'),
    path('object_data/bug_client_list/', object_data_views.bug_client_list, name='bug_client_list'),
    path('object_data/<destination>/<location_id>/bug_list/', object_data_views.bug_list, name='bug_list'),
    path('object_data/<destination>/<location_id>/customer_list/', object_data_views.customer_list,
         name='customer_list'),
    path('object_data/<destination>/<location_id>/customer_list_all/', object_data_views.customer_list_all,
         name='customer_list_all'),
    path('object_data/<destination>/<location_id>/group_list/', object_data_views.group_list, name='group_list'),
    path('object_data/<destination>/<location_id>/group_list_all/', object_data_views.group_list_all,
         name='group_list_all'),
    path('object_data/<destination>/<location_id>/<object_lookup>/link_list/', object_data_views.link_list,
         name='link_list'),
    path('object_data/<destination>/<location_id>/note_list/', object_data_views.note_list, name='note_list'),
    path('object_data/<destination>/<location_id>/object_link_list/', object_data_views.object_link_list,
         name='object_link_list'),  # WTF - Please check to make sure we need this function?
    path('object_data/<destination>/<location_id>/query_bug_client/', object_data_views.query_bug_client,
         name='query_bug_client'),
    path('object_data/<destination>/<location_id>/user_list/', object_data_views.user_list, name='user_list'),
    path('object_data/<destination>/<location_id>/user_list_all/', object_data_views.user_list_all,
         name='user_list_all'),
    path('object_data/lead_user_list/', object_data_views.lead_user_list, name='lead_user_list'),

    # Organisation
    path('organisation_duplicates/', organisation_views.organisation_duplicates, name='organisation_duplicates'),
    path('organisation_information/<int:organisation_id>/', organisation_views.organisation_information,
         name='organisation_information'),
    path('organisation_information/<int:organisation_id>/save/', organisation_views.organisation_information_save,
         name='organisation_information_save'),
    path('organisation_information/<int:organisation_id>/update_profile/',
         organisation_views.organisation_update_profile, name='organisation_update_profile'),

    # Projects
    path('project_information/<int:project_id>/', project_views.project_information, name='project_information'),
    path('project_information/<int:project_id>/save/', project_views.project_information_save,
         name='project_information_save'),

    # Requirements
    path('requirement_information/<int:requirement_id>/', requirement_views.requirement_information,
         name='requirement_information'),
    path('requirement_information/<int:requirement_id>/add_link/', requirement_views.add_requirement_link,
         name='add_requirement_link'),
    path('requirement_information/<int:requirement_id>/data/item_links/', requirement_views.get_requirement_item_links,
         name='get_requirement_item_links'),
    path('requirement_information/<int:requirement_id>/data/items/', requirement_views.get_requirement_items,
         name='get_requirement_items'),
    path('requirement_information/<int:requirement_id>/data/item_status/',
         requirement_views.get_requirement_item_status_list, name='get_requirement_item_status_list'),
    path('requirement_information/<int:requirement_id>/data/item_type/',
         requirement_views.get_requirement_item_type_list, name='get_requirement_item_type_list'),
    path('requirement_information/<int:requirement_id>/data/links/', requirement_views.get_requirement_links_list,
         name='get_requirement_links_list'),
    path('requirement_information/<int:requirement_id>/save/', requirement_views.requirement_information_save,
         name='requirement_information_save'),

    # Requirement Items
    path('requirement_item_information/<int:requirement_item_id>/', requirement_item_views.requirement_item_information,
         name='requirement_item_information'),
    path('requirement_item_information/<int:requirement_item_id>/add_link/',
         requirement_item_views.add_requirement_item_link, name='add_requirement_item_link'),
    path('requirement_item_information/<int:requirement_item_id>/', requirement_item_views.requirement_item_information,
         name='requirement_item_information'),
    path('requirement_item_information/<int:requirement_item_id>/data/links/',
         requirement_item_views.get_requirement_item_links_list, name='get_requirement_item_links_list'),
    path('requirement_item_information/<int:requirement_item_id>/save/',
         requirement_item_views.requirement_information_save, name='requirement_information_save'),

    # Request for Change
    path('rfc_deployment/<int:rfc_id>/', request_for_change_views.rfc_deployment, name='rfc_deployment'),
    path('rfc_information/<int:rfc_id>/', request_for_change_views.rfc_information, name='rfc_information'),
    path('rfc_information/<int:rfc_id>/change_task_list/', request_for_change_views.rfc_change_task_list,
         name='rfc_change_task_list'),
    path('rfc_information/<int:rfc_id>/new_change_task/', request_for_change_views.rfc_new_change_task,
         name='rfc_new_change_task'),
    path('rfc_information/<int:rfc_id>/save/', request_for_change_views.rfc_information_save,
         name='rfc_information_save'),
    path('rfc_information/<int:rfc_id>/save/backout/', request_for_change_views.rfc_save_backout,
         name='rfc_save_backout'),
    path('rfc_information/<int:rfc_id>/save/implementation/', request_for_change_views.rfc_save_implementation,
         name='rfc_save_implementation'),
    path('rfc_information/<int:rfc_id>/save/risk/', request_for_change_views.rfc_save_risk, name='rfc_save_risk'),
    path('rfc_information/<int:rfc_id>/save/test/', request_for_change_views.rfc_save_test, name='rfc_save_test'),
    path('rfc_information/<int:rfc_id>/update_status/', request_for_change_views.rfc_update_status,
         name='rfc_update_status'),
    path('rfc_readonly/<int:rfc_id>/', request_for_change_views.rfc_readonly, name='rfc_readonly'),

    # Search Items
    path('search/', search_views.search, name='search'),
    path('search/data/', search_views.search_data, name='search_data'),
    path('search/group/', search_views.search_group, name='search_group'),
    path('search/group/data/', search_views.search_group_data, name='search_group_data'),
    path('search/customer/', search_views.search_customer, name='search_customer'),
    path('search/customer/data/', search_views.search_customer_data, name='search_customer_data'),
    path('search/organisation/', search_views.search_organisation, name='search_organisation'),
    path('search/organisation/data/', search_views.search_organisation_data, name='search_organisation_data'),
    path('search/permission_set/', search_views.search_permission_set, name='search_permission_set'),
    path('search/permission_set/data/', search_views.search_permission_set_data, name='search_permission_set_data'),
    path('search/user/', search_views.search_user, name='search_user'),
    path('search/user/data/', search_views.search_user_data, name='search_user_data'),

    # Tasks
    path('task_information/<int:task_id>/', task_views.task_information, name='task_information'),
    path('task_information/<int:task_id>/save/', task_views.task_information_save, name='task_information_save'),

    # Users
    path('user_information/<int:username>/', user_views.user_information, name='user_information'),
    path('user_information/<int:username>/save/', user_views.user_information_save, name='user_information_save'),

    # Changing and Resetting Passwords
    # path('change-password/', auth_views.PasswordChangeView.as_view()),
    path(
        'password_reset/',
        auth_views.PasswordResetView.as_view(
            template_name='NearBeach/authentication/password_reset.html',
            email_template_name='NearBeach/authentication/password_reset_email.html',
        ),
        name='password_reset',
    ),
    path(
        'password_reset/done/',
        auth_views.PasswordResetDoneView.as_view(
            template_name='NearBeach/authentication/password_reset_done.html',
        ),
        name='password_reset_done',
    ),
    path(
        'reset/<uidb64>/<token>/',
        auth_views.PasswordResetConfirmView.as_view(
            template_name='NearBeach/authentication/reset.html',
        ),
        name='password_reset_confirm',
    ),
    path(
        'reset/done',
        auth_views.PasswordResetCompleteView.as_view(
            template_name='NearBeach/authentication/reset_done.html',
        ),
        name='password_reset_complete'
    ),
]

handler403 = error_views.error_403
handler404 = error_views.error_404
handler500 = error_views.error_500

"""
	path('change-password/', auth_views.PasswordChangeView.as_view()),
	path(
		'password_reset/',
		auth_views.PasswordResetView.as_view(
			template_name='NearBeach/password_reset.html',
		),
		name='password_reset',
	),
	path(
		'password_reset/done/',
		auth_views.PasswordResetDoneView.as_view(
			template_name='NearBeach/password_reset_done.html',
		),
		name='password_reset_done',
	),
	path(
		'reset/<uidb64>/<token>/',
		auth_views.PasswordResetConfirmView.as_view(
			template_name='NearBeach/reset.html',
		),
		name='password_reset_confirm',
	),
	path(
		'reset/done',
		auth_views.PasswordResetCompleteView.as_view(
			template_name='NearBeach/reset_done.html',
		),
		name='password_reset_complete'
	),



]


"""



from __future__ import unicode_literals
from django import forms
from django.forms import ModelForm

# Import from Models
from .models import *


class AddBugForm(forms.Form):
    bug_client = forms.ModelChoiceField(
        required=True,
        queryset=bug_client.objects.filter(
            is_deleted=False,
        )
    )
    bug_id = forms.IntegerField(
        required=True,
        min_value=0,
    )
    bug_description = forms.CharField(
        required=True,
    )
    bug_status = forms.CharField(
        required=True,
    )


class AddCustomerForm(forms.Form):
    customer = forms.ModelChoiceField(
        required=True,
        queryset=customer.objects.all(),
    )


class AddFolderForm(forms.Form):
    folder_description = forms.CharField(
        required=True,
        max_length=50,
    )
    parent_folder = forms.ModelChoiceField(
        required=False,
        queryset=folder.objects.all(),
    )


class AddGroupForm(forms.Form):
    group_list = forms.ModelMultipleChoiceField(
        required=True,
        queryset=group.objects.all(),
    )


class AddKanbanLinkForm(forms.Form):
    project = forms.ModelChoiceField(
        required=False,
        queryset=project.objects.filter(
            is_deleted=False,
        )
    )
    requirement = forms.ModelChoiceField(
        required=False,
        queryset=requirement.objects.filter(
            is_deleted=False,
        )
    )
    task = forms.ModelChoiceField(
        required=False,
        queryset=task.objects.filter(
            is_deleted=False,
        )
    )
    kanban_column = forms.ModelChoiceField(
        required=True,
        queryset=kanban_column.objects.filter(
            is_deleted=False,
        )
    )
    kanban_level = forms.ModelChoiceField(
        required=True,
        queryset=kanban_level.objects.filter(
            is_deleted=False,
        )
    )


class AddLinkForm(forms.Form):
    document_description = forms.CharField(
        max_length=50,
        required=True,
    )
    document_url_location = forms.URLField(
        required=True,
    )
    parent_folder = forms.ModelChoiceField(
        required=False,
        queryset=folder.objects.all(),
    )


class AddNoteForm(forms.Form):
    note = forms.CharField(
        required=True,
    )


class AddObjectLinkForm(forms.Form):
    project = forms.ModelMultipleChoiceField(
        queryset=project.objects.all(),
        required=False,
    )
    requirement = forms.ModelMultipleChoiceField(
        queryset=requirement.objects.all(),
        required=False,
    )
    requirement_item = forms.ModelMultipleChoiceField(
        queryset=requirement_item.objects.all(),
        required=False,
    )
    task = forms.ModelMultipleChoiceField(
        queryset=task.objects.all(),
        required=False,
    )


class AddRequirementLinkForm(forms.Form):
    # One external field
    project = forms.ModelMultipleChoiceField(
        required=False,
        queryset=project.objects.filter(
            is_deleted=False,
        )
    )
    task = forms.ModelMultipleChoiceField(
        required=False,
        queryset=task.objects.filter(
            is_deleted=False,
        )
    )


class AdminAddUserForm(forms.Form):
    group = forms.ModelMultipleChoiceField(
        queryset=group.objects.all(),
    )
    permission_set = forms.ModelMultipleChoiceField(
        queryset=permission_set.objects.all(),
    )
    username = forms.ModelChoiceField(
        queryset=User.objects.all(),
    )


class ChangeTaskStatusForm(forms.ModelForm):
    class Meta:
        model = change_task
        fields = [
            'change_task_status',
        ]


class CheckKanbanBoardName(forms.Form):
    kanban_board_name = forms.CharField(max_length=255)


class CustomerForm(forms.ModelForm):
    # Basic Meta Data
    class Meta:
        model = customer
        fields = [
            'customer_title',
            'customer_first_name',
            'customer_last_name',
            'customer_email',
            'organisation',
        ]


class DocumentUploadForm(ModelForm):
    document = forms.FileField(
        required=True,
    )
    parent_folder = forms.ModelChoiceField(
        required=False,
        queryset=folder.objects.all(),
    )

    class Meta:
        model = document
        fields = {
            'document',
            'document_description',
        }


class KanbanCardForm(forms.ModelForm):
    kanban_card_id = forms.ModelChoiceField(
        required=True,
        queryset=kanban_card.objects.all(),
    )
    kanban_card_description = forms.CharField(
        required=False,
    )

    class Meta:
        model = kanban_card
        fields = {
            'kanban_card_id',
            'kanban_card_text',
            'kanban_card_description',
        }


class AddUserForm(forms.Form):
    user_list = forms.ModelMultipleChoiceField(
        required=True,
        queryset=User.objects.all(),
    )


class LoginForm(forms.Form):
    username = forms.CharField(
        widget=forms.TextInput(attrs={
            'placeholder': 'Username',
            'class': 'form-control',
            'required': True,
            'autofocus': True,
        })
    )
    password = forms.CharField(
        widget=forms.PasswordInput(attrs={
            'placeholder': 'Password',
            'class': 'form-control',
            'required': True,
        })
    )


class MoveKanbanCardForm(forms.Form):
    # Get Query Sets
    kanban_column_results = kanban_column.objects.all()
    kanban_level_results = kanban_level.objects.all()

    # New card information
    new_card_column = forms.ModelChoiceField(
        required=True,
        queryset=kanban_column_results,
    )
    new_card_level = forms.ModelChoiceField(
        required=True,
        queryset=kanban_level_results,
    )
    new_card_sort_number = forms.IntegerField()

    # Old card information
    old_card_column = forms.ModelChoiceField(
        required=True,
        queryset=kanban_column_results,
    )
    old_card_level = forms.ModelChoiceField(
        required=True,
        queryset=kanban_level_results,
    )
    old_card_sort_number = forms.IntegerField()


class NewChangeTaskForm(forms.ModelForm):
    # Basic Meta Data
    class Meta:
        model = change_task
        fields = [
            'request_for_change',
            'change_task_title',
            'change_task_description',
            'change_task_start_date',
            'change_task_end_date',
            'change_task_seconds',
            # 'change_task_assigned_user',
            # 'change_task_qa_user',
            'change_task_required_by',
            'is_downtime',
        ]


class NewCustomerForm(forms.ModelForm):
    organisation = forms.ModelChoiceField(
        queryset=organisation.objects.all(),
        required=False,
    )

    # Basic Meta Data
    class Meta:
        model = customer
        fields = [
            'customer_title',
            'customer_first_name',
            'customer_last_name',
            'customer_email',
            'organisation',
        ]


class NewGroupForm(forms.ModelForm):
    parent_group = forms.ModelChoiceField(
        queryset=group.objects.all(),
        required=False,
    )

    # Basic Meta Data
    class Meta:
        model = group
        fields = [
            'group_name',
            'parent_group',
        ]


class NewKanbanCardForm(forms.ModelForm):
    kanban_card_description = forms.CharField(
        required=False,
    )

    # Basic Meta Data
    class Meta:
        model = kanban_card
        fields = [
            'kanban_card_text',
            'kanban_card_description',
            'kanban_level',
            'kanban_column',
        ]


class NewKanbanForm(forms.ModelForm):
    column_title = forms.SelectMultiple()
    level_title = forms.SelectMultiple()

    # Basic Meta Data
    class Meta:
        model = kanban_board
        fields = [
            'kanban_board_name',
        ]


class NewPermissionSetForm(forms.ModelForm):
    # Basic Meta Data
    class Meta:
        model = permission_set
        fields = [
            'permission_set_name',
        ]

class NewProjectForm(forms.ModelForm):
    project_start_date = forms.DateTimeField(
        input_formats=['c'],
    )
    project_end_date = forms.DateTimeField(
        input_formats=['c'],
    )
    group_list = forms.ModelMultipleChoiceField(
        required=True,
        queryset=group.objects.filter(
            is_deleted=False,
        )
    )

    # Basic Meta Data
    class Meta:
        model = project
        fields = [
            'project_name',
            'project_description',
            'project_start_date',
            'project_end_date',
            'organisation',
        ]


class NewRequestForChangeForm(forms.ModelForm):
    group_list = forms.ModelMultipleChoiceField(
        required=True,
        queryset=group.objects.filter(
            is_deleted=False,
        )
    )

    # Basic Meta Data
    class Meta:
        model = request_for_change
        fields = [
            'rfc_title',
            'rfc_summary',
            'rfc_type',
            'rfc_implementation_start_date',
            'rfc_implementation_end_date',
            'rfc_implementation_release_date',
            'rfc_version_number',
            'rfc_lead',
            'rfc_priority',
            'rfc_risk',
            'rfc_impact',
            'rfc_risk_and_impact_analysis',
            'rfc_implementation_plan',
            'rfc_backout_plan',
            'rfc_test_plan',
        ]


class NewTaskForm(forms.ModelForm):
    task_start_date = forms.DateTimeField(
        input_formats=['c'],
    )
    task_end_date = forms.DateTimeField(
        input_formats=['c'],
    )
    group_list = forms.ModelMultipleChoiceField(
        required=True,
        queryset=group.objects.filter(
            is_deleted=False,
        )
    )

    # Basic Meta Data
    class Meta:
        model = task
        fields = [
            'task_short_description',
            'task_long_description',
            'task_start_date',
            'task_end_date',
            'organisation',
        ]


class NewUserForm(forms.ModelForm):
    password1 = forms.CharField(
        max_length=255,
        required=True,
    )
    password2 = forms.CharField(
        max_length=255,
        required=True,
    )

    # Basic Meta data
    class Meta:
        model = User
        fields = [
            'username',
            'first_name',
            'last_name',
            'email',
        ]


class OrganisationForm(forms.ModelForm):
    # Basic Meta data
    class Meta:
        model = organisation
        fields = [
            'organisation_name',
            'organisation_website',
            'organisation_email',
        ]


class OrganisationProfilePictureForm(forms.ModelForm):
    # Basic Meta Data
    class Meta:
        fields = [
            'organisation_picture'
        ]


class PermissionSetForm(forms.ModelForm):
    class Meta:
        model = permission_set
        exclude = [
            'change_user',
            'is_deleted',
        ]


class ProjectForm(forms.ModelForm):
    project_start_date = forms.DateTimeField(
        input_formats=['c'],
    )
    project_end_date = forms.DateTimeField(
        input_formats=['c'],
    )

    # Basic Meta Data
    class Meta:
        model = project
        fields = [
            'project_name',
            'project_description',
            'project_start_date',
            'project_end_date',
        ]


class NewRequirementItemForm(forms.ModelForm):
    # Basic Meta data
    class Meta:
        model = requirement_item
        fields = [
            'requirement_item_title',
            'requirement_item_scope',
            'requirement_item_status',
            'requirement_item_type',
        ]


class NewRequirementForm(forms.ModelForm):
    # One external field
    group_list = forms.ModelMultipleChoiceField(
        required=True,
        queryset=group.objects.filter(
            is_deleted=False,
        )
    )

    # Basic Meta data
    class Meta:
        model = requirement
        fields = [
            'requirement_title',
            'requirement_scope',
            'requirement_status',
            'requirement_type',
            'organisation',
        ]


class QueryBugClientForm(forms.Form):
    bug_client_id = forms.ModelChoiceField(
        required=True,
        queryset=bug_client.objects.filter(
            is_deleted=False,
        )
    )
    search = forms.CharField(
        max_length=50,
    )


class RfcModuleForm(forms.Form):
    # This form is for all the sub modules that need to be saved separately.
    text_input = forms.CharField(
        required=True,
    )
    priority_of_change = forms.IntegerField(
        required=False,
    )
    risk_of_change = forms.IntegerField(
        required=False,
    )
    impact_of_change = forms.IntegerField(
        required=False,
    )


class RfcInformationSaveForm(forms.ModelForm):
    class Meta:
        model = request_for_change
        fields = [
            'rfc_title',
            'rfc_summary',
            'rfc_type',
            'rfc_version_number',
            'rfc_implementation_start_date',
            'rfc_implementation_end_date',
            'rfc_implementation_release_date',
        ]


class SearchForm(forms.Form):
    # Just have a simple search field
    search = forms.CharField(
        required=False,
    )


class SearchObjectsForm(forms.Form):
    include_closed = forms.BooleanField(
        required=False,
        initial=False,
    )
    search = forms.CharField(
        required=False,
    )


class TaskInformationForm(forms.ModelForm):
    # Basic Meta data
    class Meta:
        model = task
        fields = [
            'task_short_description',
            'task_long_description',
            'task_start_date',
            'task_end_date',
        ]


class UpdateRequirementForm(forms.ModelForm):
    # Basic Meta data
    class Meta:
        model = requirement
        fields = [
            'requirement_title',
            'requirement_scope',
            'requirement_status',
            'requirement_type',
        ]


class UpdateRequirementItemForm(forms.ModelForm):
    # Basic Meta data
    class Meta:
        model = requirement_item
        fields = [
            'requirement_item_title',
            'requirement_item_scope',
            'requirement_item_status',
            'requirement_item_type',
        ]


class UpdateRFCStatus(forms.ModelForm):
    # Basic Meta Data
    class Meta:
        model = request_for_change
        fields = [
            'rfc_status',
        ]


class UpdateUserForm(forms.ModelForm):
    # Basic Meta Data
    class Meta:
        model = User
        fields = [
            'first_name',
            'last_name',
            'email',
            'is_active',
            'is_superuser',
        ]



from NearBeach.models import *


# Internal function
def get_object_from_destination(input_object,destination,location_id):
    """
    To stop the repeat code of finding specific objects using destination and location_id - we will import
    the object filter for it here - before returning it.
    :param object: The object we want to filter
    :param destination: The destination we are interested in
    :param location_id: The location_id
    :return:
    """
    if destination == "kanban_board":
        input_object = input_object.filter(
            kanban_board_id=location_id,
        )
    if destination == "kanban_card":
        input_object = input_object.filter(
            kanban_card_id=location_id,
        )
    elif destination == "opportunity":
        input_object = input_object.filter(
            opportunity_id=location_id,
        )
    elif destination == "organisation":
        input_object = input_object.filter(
            organisation_id=location_id,
        )
    elif destination == "project":
        input_object = input_object.filter(
            project_id=location_id,
        )
    elif destination == "quote":
        input_object = input_object.filter(
            quote_id=location_id,
        )
    elif destination == "requirement":
        input_object = input_object.filter(
            requirement_id=location_id,
        )
    elif destination == "request_for_change":
        input_object = input_object.filter(
            request_for_change_id=location_id,
        )
    elif destination == "requirement_item":
        input_object = input_object.filter(
            requirement_item_id=location_id,
        )
    elif destination == "task":
        input_object = input_object.filter(
            task_id=location_id,
        )
    elif destination == "whiteboard":
        input_object = input_object.filter(
            whiteboard_id=location_id,
        )

    # Just send back the array
    return input_object


# Internal function
def set_object_from_destination(input_object,destination,location_id):
    """
    This function is used to set data against an object using the destination and location data.
    :param input_object: The input object that we are setting data for
    :param destination: The destination we are interested in
    :param location_id: The location we are interested in
    :return:
    """
    if destination == "kanban_board":
        input_object.kanban_board = kanban_board.objects.get(kanban_board_id=location_id)
    if destination == "kanban_card":
        input_object.kanban_card = kanban_card.objects.get(kanban_card_id=location_id)
    elif destination == "opportunity":
        input_object.opportunity = opportunity.objects.get(object_id=location_id)
    elif destination == "organisation":
        input_object.organisation = organisation.objects.get(organisation_id=location_id)
    elif destination == "quote":
        input_object.quote = quote.objects.get(quote_id=location_id)
    if destination == "project":
        input_object.project = project.objects.get(project_id=location_id)
    elif destination == "request_for_change":
        input_object.request_for_change = request_for_change.objects.get(request_for_change_id=location_id)
    elif destination == "requirement":
        input_object.requirement = requirement.objects.get(requirement_id=location_id)
    elif destination == "requirement_item":
        input_object.requirement_item = requirement_item.objects.get(requirement_item_id=location_id)
    elif destination == "task":
        input_object.task = task.objects.get(task_id=location_id)
    elif destination == "whiteboard":
        input_object.whiteboard = whiteboard.objects.get(white_board_id=location_id)

    # Return what we have
    return input_object



from django.contrib.auth.decorators import login_required
from django.core import serializers
from django.conf import settings
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseBadRequest, FileResponse, JsonResponse
from django.db.models import Q
from django.utils.encoding import smart_str
from django.views.decorators.http import require_http_methods
from django.template import loader
from NearBeach.views.tools.internal_functions import *
from django.core.serializers.json import DjangoJSONEncoder

from ..forms import *

import json, os


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def document_add_folder(request,destination,location_id):
    """
    This will add a folder to the user's destination and location_id
    :param request:
    :param destination:
    :param location_id:
    :return:
    """

    # Check users PERMISSION - NEED TO ADD

    # Obtain the data and verify against the form
    form = AddFolderForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Save the form information
    folder_submit = folder(
        change_user=request.user,
        folder_description=form.cleaned_data['folder_description'],
        parent_folder=form.cleaned_data['parent_folder'],
    )
    folder_submit = set_object_from_destination(folder_submit,destination,location_id)
    folder_submit.save()

    # Return the data back
    folder_results = folder.objects.filter(folder_id=folder_submit.folder_id)

    return HttpResponse(serializers.serialize('json',folder_results),content_type='application/json')


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def document_add_link(request,destination,location_id):
    """

    :param request:
    :param destination:
    :param location_id:
    :return:
    """
    # ADD IN PERMISSION CHECKS

    # Get the form data
    form = AddLinkForm(request.POST)
    if not form.is_valid():
        print(form.errors)
        return HttpResponseBadRequest(form.errors)

    # Save the document link
    document_submit = document(
        change_user=request.user,
        document_description=form.cleaned_data['document_description'],
        document_url_location=form.cleaned_data['document_url_location'],
    )
    document_submit.save()

    # Save the document permissions
    document_permission_submit = document_permission(
        change_user=request.user,
        document_key=document_submit,
        folder=form.cleaned_data['parent_folder'],
    )
    document_permission_submit = set_object_from_destination(
        document_permission_submit,
        destination,
        location_id
    )
    document_permission_submit.save()

    # Get current document results to send back
    document_results = document_permission.objects.filter(
        is_deleted=False,
        document_key=document_submit,
    ).values(
        'document_key__document_description',
        'document_key__document_url_location',
        'document_key__document',
        'document_key__whiteboard',
        'folder',
    )

    # Send back json data
    json_results = json.dumps(list(document_results), cls=DjangoJSONEncoder)

    return HttpResponse(json_results, content_type='application/json')


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def document_list_files(request,destination,location_id):
    """
    Get the documents that are associated with the destination and location id

    Method
    ~~~~~~
    1. Get all the document permissions connected to the destination and location
    2. Get the document information from this
    :param request:
    :param destination:
    :param location_id:
    :return:
    """
    document_permission_results = document_permission.objects.filter(
        is_deleted=False,
    ).values(
        'document_key_id',
        'document_key__document_description',
        'document_key__document_url_location',
        'document_key__document',
        'document_key__whiteboard',
        'folder',
    )

    # Limit to the required destination and location id
    document_permission_results = get_object_from_destination(
        document_permission_results,
        destination,
        location_id
    )

    # Send back json data
    json_results = json.dumps(list(document_permission_results), cls=DjangoJSONEncoder)

    return HttpResponse(json_results, content_type='application/json')


    # # Get the document information
    # document_results = document.objects.filter(
    #     is_deleted=False,
    #     document_key__in=document_permission_results.values('document_key')
    # )
    #
    # return HttpResponse(serializers.serialize('json',document_results),content_type='application/json')


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def document_list_folders(request,destination,location_id):
    """
    Get the folders that are associated with the destination and location id

    Method
    ~~~~~~
    1. Get all the document permissions connected to the destination and location
    2. Get the folder information from this

    :param request:
    :param destination:
    :param location_id:
    :return:
    """
    # Get the document information
    folder_results = folder.objects.filter(
        is_deleted=False,
    )
    folder_results = get_object_from_destination(folder_results,destination,location_id)

    return HttpResponse(serializers.serialize('json',folder_results),content_type='application/json')


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def document_upload(request,destination,location_id):
    """
    The following function will deal with the uploaded document. It will first;
    1. Check user's permission
    2. Place the request.POST values into a form for validation
    3. Create a new row in document and store the relevant data inside
    4. Create a new row in document permission and assign it to the appropriate object
    5. Send back the document information in JSON format
    :param request:
    :param destination: the object in question
    :param location_id: The location of the object
    :param folder_id: Which folder we will associate this with
    :return:
    """

    # WRITE CODE TO CHECK THE USER'S PERMISSION

    form = DocumentUploadForm(request.POST,request.FILES)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Get the document description - if blank we use the file name
    document_description = form.cleaned_data['document_description']
    file = form.cleaned_data['document']
    if document_description == "":
        # Replace the document description with the file name
        document_description = str(file)

    # Add the document row
    document_submit = document(
        change_user=request.user,
        document_description=document_description,
        document=file,
    )
    document_submit.save()

    # Add the document permission row
    document_permission_submit = document_permission(
        change_user=request.user,
        document_key=document_submit,
        folder=form.cleaned_data['parent_folder'],
    )
    document_permission_submit = set_object_from_destination(
        document_permission_submit,
        destination,
        location_id
    )
    document_permission_submit.save()

    # Get current document results to send back
    document_results = document_permission.objects.filter(
        is_deleted=False,
        document_key=document_submit,
    ).values(
        'document_key_id',
        'document_key__document_description',
        'document_key__document_url_location',
        'document_key__document',
        'document_key__whiteboard',
        'folder',
    )

    # Handle the document upload
    handle_file_upload(request.FILES['document'],document_results)

    # Send back json data
    json_results = json.dumps(list(document_results), cls=DjangoJSONEncoder)

    return HttpResponse(json_results, content_type='application/json')


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def get_max_upload(request):
    """
    This function will query the settings file for the variable "max_upload_size". If it does not exist it will send back
    a simple 0. If it does exist it will send back itself
    :param request:
    :return:
    """
    if hasattr(settings, 'MAX_UPLOAD_SIZE'):
        send_value = {'max_upload_size': settings.MAX_UPLOAD_SIZE}
    else:
        max_upload_size = {'max_upload_size': 0}

    return JsonResponse(max_upload_size)


@login_required(login_url='login',redirect_field_name="")
def private_download_file(request,document_key):
    """
    The following function will check;
    1. The user's permission to the document
    2. If the document exists

    From there it will use X-Sendfile to send the document.
    :param request:
    :param document_key:
    :return:
    """
    # Extract the user groups the user is associated with
    user_group_results = user_group.objects.filter(
        is_deleted=False,
        username=request.user,
    )

    # Extract the permissions the document is associated with
    document_permission_results = document_permission.objects.filter(
        is_deleted=False,
        document_key=document_key,
    )

    # Consolidate at the object assignment
    object_assignment_results = object_assignment.objects.filter(
        Q(
            is_deleted=False,
            # JOIN IN USER GROUPS
            group_id__in=user_group_results.values('group_id')
        ) &
        Q(
            # One of these conditions have to be met
            Q(
                # If the document is uploaded against a customer
                customer__isnull=False,
            ) |
            Q(
                # If the document is uploaded against an organisation
                organisation__isnull=False,
            ) |
            Q(
                # Project Links
                project__isnull=False,
                project_id__in=document_permission_results.values('project_id')
            ) |
            Q(
                # Task links
                task__isnull=False,
                task_id__in=document_permission_results.values('task_id')
            ) |
            Q(
                # Whiteboard
                whiteboard__isnull=False,
                whiteboard_id__in=document_permission_results.values('whiteboard_id')
            ) |
            Q(
                # Requirement
                requirement__isnull=False,
                requirement_id__in=document_permission_results.values('requirement_id')
            ) |
            Q(
                # Requirement Item
                # Have to use the requirement item's requirement as permissions are not on the item
                requirement_id__in=requirement_item.objects.filter(
                    is_deleted=False,
                    requirement_item_id__in=document_permission_results.values('requirement_item_id'),
                    requirement_item_id__isnull=False,
                ).values('requirement_id')
            ) |
            Q(
                # Request for change
                request_for_change__isnull=False,
                request_for_change_id__in=document_permission_results.values('request_for_change_id')
            )
        )
    )

    # If the object_assignment_results.count() == 0, then user does not have permissions
    if object_assignment_results.count() == 0 & request.user.is_superuser == False:
        return HttpResponseBadRequest("Sorry - there is no document")

    # Get Document information
    document_results = document.objects.get(document_key=document_key)  # Need to change this to a 404

    # If not a document but a URL
    if document_results.document_url_location:
        return HttpResponseRedirect(document_results.document_url_location)

    # Get the Document path information
    path = '%s/%s/%s' % (
        settings.PRIVATE_MEDIA_ROOT,
        document_key,
        document_results.document
    )

    # Send file to user
    return FileResponse(open(path, 'rb'))


#Internal Function
def handle_file_upload(upload_document,document_results):
    """
    This function will upload the file and store it in the private folder destination under a subfolder that contains
    the same document_key value.
    :param upload_document: The FILE itself - to be uploaded
    :param document_results: The document_results - with variables we require
    :return:
    """
    # Make the directory we want to save the file in. The directory will have the document_key
    file_permissions = 0o755 #Look at these permissions later
    path = os.path.join(
        settings.PRIVATE_MEDIA_ROOT,
        '%s' % (document_results[0]['document_key_id'],),
    )
    os.mkdir(path, file_permissions)

    storage_location = '%s/%s/%s' % (
        settings.PRIVATE_MEDIA_ROOT,
        document_results[0]['document_key_id'],
        #document_results[0]['document_key__document_description'],
        document_results[0]['document_key__document'],
    )

    #Save the upload document in the location
    with open(storage_location,'wb+') as destination:
        for chunk in upload_document.chunks():
            destination.write(chunk)


import urllib

from django.contrib.auth.decorators import login_required
from NearBeach.models import *
from django.core import serializers
from django.core.serializers.json import DjangoJSONEncoder
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_http_methods
from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseRedirect, JsonResponse
from django.urls import reverse
from django.template import loader
from django.db.models import Sum, Q, Min
from NearBeach.forms import *
from NearBeach.views.tools.internal_functions import *
from NearBeach.user_permissions import return_user_permission_level
import json, urllib3


@login_required(login_url='login',redirect_field_name="")
def new_task(request):
    """

    :param request:
    :return:
    """
    # ADD IN PERMISSIONS CHECKER

    # Template
    t = loader.get_template('NearBeach/tasks/new_task.html')

    # Get data
    group_results = group.objects.filter(
        is_deleted=False,
    )

    # Context
    c = {
        'group_results': serializers.serialize('json',group_results),
    }

    return HttpResponse(t.render(c,request))


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name='')
def new_task_save(request):
    """

    :param request:
    :return:
    """

    # ADD IN USER PERMISSIONS

    # Get form data
    form = NewTaskForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Create the new task
    task_submit = task(
        change_user=request.user,
        creation_user=request.user,
        task_short_description=form.cleaned_data['task_short_description'],
        task_long_description=form.cleaned_data['task_long_description'],
        task_start_date=form.cleaned_data['task_start_date'],
        task_end_date=form.cleaned_data['task_end_date'],
        organisation=form.cleaned_data['organisation'],
    )
    task_submit.save()

    # Get the group list and apply the permissions
    group_list = request.POST.getlist("group_list")

    for single_group in group_list:
        # Get the group instance
        group_instance = group.objects.get(group_id=single_group)

        # Save the group instance against object assignment
        submit_object_assignment = object_assignment(
            group_id=group_instance,
            task=task_submit,
            change_user=request.user,
        )

        # Save
        submit_object_assignment.save()

    # Send back requirement_information URL
    return HttpResponse(reverse('task_information', args={task_submit.task_id}))


@login_required(login_url='login',redirect_field_name="")
def task_information(request,task_id):
    """

    :param request:
    :param task_id:
    :return:
    """
    # ADD IN PERMISSIONS CHECKER

    # Template
    t = loader.get_template('NearBeach/tasks/task_information.html')

    # Get Data
    task_results = task.objects.get(task_id=task_id)

    organisation_results = organisation.objects.filter(
        is_deleted=False,
        organisation_id=task_results.organisation_id,
    )

    # Context
    c = {
        'organisation_results': serializers.serialize('json',organisation_results),
        'task_id': task_id,
        'task_results': serializers.serialize('json',[task_results]),
    }

    return HttpResponse(t.render(c,request))


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name='')
def task_information_save(request,task_id):
    """

    :param request:
    :param task_id:
    :return:
    """

    # CHECK USER PERMISSIONS

    # Form
    form = TaskInformationForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Get the instance
    update_task = task.objects.get(task_id=task_id)

    # Update the values
    update_task.task_short_description = form.cleaned_data['task_short_description']
    update_task.task_long_description = form.cleaned_data['task_long_description']
    update_task.task_start_date = form.cleaned_data['task_start_date']
    update_task.task_end_date = form.cleaned_data['task_end_date']

    update_task.save()

    return HttpResponse("")




from django.http import HttpResponse, HttpResponseBadRequest
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_http_methods

from NearBeach.forms import ChangeTaskStatusForm
from NearBeach.models import change_task


@require_http_methods(['POST'])
@login_required(login_url='login', redirect_field_name="")
def update_status(request,change_task_id):
    """

    :param request:
    :param change_task_id:
    :return:
    """

    # Get form data
    form = ChangeTaskStatusForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Get change task
    change_task_results = change_task.objects.get(change_task_id=change_task_id)

    # Update the change task results
    change_task_results.change_task_status = form.cleaned_data['change_task_status']
    change_task_results.save()

    return HttpResponse("")



from django.core import serializers
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_http_methods
from django.http import HttpResponseBadRequest
from django.urls import reverse
from django.template import loader
from NearBeach.forms import *
from NearBeach.views.tools.internal_functions import *
from NearBeach.decorators.check_user_permissions import project_permissions


@login_required(login_url='login',redirect_field_name="")
@project_permissions(min_permission_level=3)
def new_project(request):
    """

    :param request:
    :return:
    """
    # ADD IN PERMISSIONS CHECKER

    # Template
    t = loader.get_template('NearBeach/projects/new_project.html')

    # Get data we require
    group_results = group.objects.filter(
        is_deleted=False,
    )

    # Context
    c = {
        'group_results': serializers.serialize('json',group_results),
    }

    return HttpResponse(t.render(c,request))


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name='')
@project_permissions(min_permission_level=3)
def new_project_save(request):
    """

    :param request:
    :return:
    """

    # CHECK USERS PERMISSION HERE

    # Get the form data
    form = NewProjectForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Create the project
    project_submit = project(
        change_user=request.user,
        creation_user=request.user,
        project_name=form.cleaned_data['project_name'],
        project_description=form.cleaned_data['project_description'],
        project_start_date=form.cleaned_data['project_start_date'],
        project_end_date=form.cleaned_data['project_end_date'],
        organisation=form.cleaned_data['organisation'],
    )
    project_submit.save()

    # Get the group list and apply the permissions
    group_list = request.POST.getlist("group_list")

    for single_group in group_list:
        # Get the group instance
        group_instance = group.objects.get(group_id=single_group)

        # Save the group instance against object assignment
        submit_object_assignment = object_assignment(
            group_id=group_instance,
            project=project_submit,
            change_user=request.user,
        )

        # Save
        submit_object_assignment.save()

    # Send back requirement_information URL
    return HttpResponse(reverse('project_information', args={project_submit.project_id}))


@login_required(login_url='login',redirect_field_name="")
@project_permissions(min_permission_level=1)
def project_information(request,project_id):
    """

    :param request:
    :param project_id:
    :return:
    """
    # ADD IN PERMISSIONS CHECKER

    # Template
    t = loader.get_template('NearBeach/projects/project_information.html')

    # Get data
    project_results = project.objects.get(project_id=project_id)

    organisation_results = organisation.objects.filter(
        is_deleted=False,
        organisation_id=project_results.organisation_id,
    )

    # Context
    c = {
        'organisation_results': serializers.serialize('json',organisation_results),
        'project_id': project_id,
        'project_results': serializers.serialize('json',[project_results]),
    }

    return HttpResponse(t.render(c,request))


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name='')
@project_permissions(min_permission_level=2)
def project_information_save(request,project_id):
    """

    :param request:
    :param project_id:
    :return:
    """

    # ADD IN PERMISSION CHECKER

    # Get the form data
    form = ProjectForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Get the project data
    project_update = project.objects.get(project_id=project_id)
    project_update.project_name = form.cleaned_data['project_name']
    project_update.project_description = form.cleaned_data['project_description']
    project_update.project_start_date = form.cleaned_data['project_start_date']
    project_update.project_end_date = form.cleaned_data['project_end_date']

    # Save
    project_update.save()

    return HttpResponse("Good")



from django.http import HttpResponse, HttpResponseBadRequest
from django.shortcuts import reverse
from django.template import loader
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_http_methods
from django.core import serializers
from django.core.serializers.json import DjangoJSONEncoder

from NearBeach.models import *
from NearBeach.forms import *

import json


@login_required(login_url='login',redirect_field_name="")
def new_permission_set(request):
    """

    :param request:
    :return:
    """

    # Check user permissions

    # Get template
    t = loader.get_template('NearBeach/permission_sets/new_permission_set.html')

    # Get context
    c = {}

    return HttpResponse(t.render(c, request))


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def new_permission_set_save(request):
    """

    :param request:
    :return:
    """

    # Check user permissions

    # Get form data
    form = NewPermissionSetForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Save the data
    submit_permission_set = permission_set(
        permission_set_name=form.cleaned_data['permission_set_name'],
        change_user=request.user,
    )

    submit_permission_set.save()

    # Return back the permission set information URL
    return HttpResponse(reverse('permission_set_information',args={submit_permission_set.permission_set_id}))


@login_required(login_url='login',redirect_field_name="")
def permission_set_information(request, permission_set_id):
    """

    :param request:
    :param permission_set_id:
    :return:
    """

    # Add in permission checks

    # Import template
    t = loader.get_template('NearBeach/permission_sets/permission_set_information.html')

    # Get data
    permission_set_results = permission_set.objects.get(permission_set_id=permission_set_id)

    user_list_results = user_group.objects.filter(
        is_deleted=False,
        permission_set_id=permission_set_id,
    ).values(
        'username',
        'username__first_name',
        'username__last_name',
        'username__email',
        'group',
        'group__group_name',
        'permission_set',
        'permission_set__permission_set_name',
    ).order_by(
        'username__first_name',
        'username__last_name',
        'permission_set__permission_set_name',
    )
    user_list_results = json.dumps(list(user_list_results), cls=DjangoJSONEncoder)

    # Create the context
    c = {
        'permission_set_results': serializers.serialize('json', [permission_set_results]),
        'user_list_results': user_list_results,
        'permission_set_id': permission_set_id,
        'permission_boolean': json.dumps(PERMISSION_BOOLEAN),
        'permission_level': json.dumps(PERMISSION_LEVEL),
    }

    return HttpResponse(t.render(c, request))


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def permission_set_information_save(request, permission_set_id):
    """

    :param request:
    :param permission_set_id:
    :return:
    """

    # ADD IN USER PERMISSIONS LATER

    # Check to make sure nothing changes for the administration permissions
    if permission_set_id == 1:
        return HttpResponseBadRequest("Error - can not edit administration")

    # Get form data
    form = PermissionSetForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Get the object
    update_permission_set = permission_set.objects.get(permission_set_id=permission_set_id)

    # Update the object
    update_permission_set.permission_set_name = form.cleaned_data['permission_set_name']
    update_permission_set.administration_assign_user_to_group = form.cleaned_data['administration_assign_user_to_group']
    update_permission_set.administration_create_group = form.cleaned_data['administration_create_group']
    update_permission_set.administration_create_permission_set = form.cleaned_data[
        'administration_create_permission_set']
    update_permission_set.administration_create_user = form.cleaned_data['administration_create_user']
    update_permission_set.bug_client = form.cleaned_data['bug_client']
    update_permission_set.customer = form.cleaned_data['customer']
    update_permission_set.kanban = form.cleaned_data['kanban']
    update_permission_set.kanban_card = form.cleaned_data['kanban_card']
    update_permission_set.organisation = form.cleaned_data['organisation']
    update_permission_set.project = form.cleaned_data['project']
    update_permission_set.requirement = form.cleaned_data['requirement']
    update_permission_set.request_for_change = form.cleaned_data['request_for_change']
    update_permission_set.task = form.cleaned_data['task']
    update_permission_set.document = form.cleaned_data['document']
    update_permission_set.kanban_comment = form.cleaned_data['kanban_comment']
    update_permission_set.project_history = form.cleaned_data['project_history']
    update_permission_set.task_history = form.cleaned_data['task_history']

    update_permission_set.save()

    return HttpResponse("")



# Import Forms
from ..forms import *

# Import Django Libraries
from django.contrib import auth
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseRedirect
from django.template import loader
from django.urls import reverse
from random import SystemRandom

# Import Python Libraries
import json, urllib.parse, random

def check_first_time_login(request):
    """
    The following function will check if it is the first time logged in by user. i.e. There are no permission sets.
    If there are no permission sets - this function will create it.

    If there are permission sets - nothing done.
    :return:

    The user has been authenticated. Now the system will store the user's permissions and group
    into cookies. :)

    First Setup
    ~~~~~~~~~~~
    If permission_set with id of 1 does not exist, go through first stage setup.
    """
    if not permission_set.objects.all():
        # Create administration permission_set
        submit_permission_set = permission_set(
            permission_set_name="Administration Permission Set",
            administration_assign_user_to_group=4,
            administration_create_group=4,
            administration_create_permission_set=4,
            administration_create_user=4,
            bug_client=4,
            customer=4,
            kanban=4,
            kanban_card=4,
            organisation=4,
            project=4,
            requirement=4,
            task=4,
            document=1,
            kanban_comment=1,
            project_history=1,
            task_history=1,
            change_user=request.user,
        )
        submit_permission_set.save()

        # Create admin group
        submit_group = group(
            group_name="Administration",
            change_user=request.user,
        )
        submit_group.save()

        # Add user to admin group
        submit_user_group = user_group(
            username=request.user,
            group=group.objects.get(group_id=1),
            permission_set=permission_set.objects.get(permission_set_id=1),
            change_user=request.user,
        )
        submit_user_group.save()

    request.session['is_superuser'] = request.user.is_superuser


def check_recaptcha(post_data):
    """
    Determine if the user has setup the recaptcha in the settings.py file, and then check the result against google.
    :param post_data:
    :return:
    """
    if hasattr(settings, 'RECAPTCHA_PUBLIC_KEY') and hasattr(settings, 'RECAPTCHA_PRIVATE_KEY'):
        RECAPTCHA_PRIVATE_KEY = settings.RECAPTCHA_PRIVATE_KEY
    else:
        #User has not setup recaptcha - return true
        return True

    """
    As the Google documentation states. I have to send the request back to
    the given URL. It gives back a JSON object, which will contain the
    success results.

    Method
    ~~~~~~
    1.) Collect the variables
    2.) With the data - encode the variables into URL format
    3.) Send the request to the given URL
    4.) The response will open and store the response from GOOGLE
    5.) The results will contain the JSON Object
    """
    recaptcha_response = post_data.get('g-recaptcha-response')
    url = 'https://www.google.com/recaptcha/api/siteverify'
    values = {
        'secret': RECAPTCHA_PRIVATE_KEY,
        'response': recaptcha_response
    }

    """
    SECURITY ISSUE
    ~~~~~~~~~~~~~~
    The URL could contain a file. Which we do not want executed by mistake. So we just make sure that the URL starts
    with a http instead of ftp or file.

    We place the  at the end of the json_data because we have checked the field. This should be just a json 
    response. If it is not at this point then it will produce a server issue.
    """
    if url.lower().startswith('http'):
        req = urllib.request.Request(url)
    else:
        raise ValueError from None

    with urllib.request.urlopen(req, urllib.parse.urlencode(values).encode('utf8')) as response:  # nosec
        result = json.load(response)

        # Check to see if the user is a robot. Success = human
    if result['success']:
        return True
    else:
        return False

def login(request):
    """
	For some reason I can not use the varable "LoginForm" here as it is already being used.
	Instead I will use the work form.

	The form is declared at the start and filled with either the POST data OR nothing. If this
	process is called in POST, then the form will be checked and if it passes the checks, the
	user will be logged in.

	If the form is not in POST (aka GET) OR fails the checks, then it will create the form with
	the relevant errors.
	"""
    form = LoginForm(request.POST or None)

    # POST
    if request.method == 'POST':
        if form.is_valid():
            # Check if user passes recaptcha
            if check_recaptcha(request.POST) == True:
                # Looks like we can authenticate the user
                username = form.cleaned_data.get("username")
                password = form.cleaned_data.get("password")

                user = auth.authenticate(username=username, password=password)
                if user is not None:
                    auth.login(request, user)

            # Just double checking. :)
            if request.user.is_authenticated:
                #Check to make sure it isn't first time login -> need to setup functionalities
                check_first_time_login(request)

                return HttpResponseRedirect(reverse('dashboard'))

    # Get recaptcha public key
    if hasattr(settings, 'RECAPTCHA_PUBLIC_KEY') and hasattr(settings, 'RECAPTCHA_PRIVATE_KEY'):
        RECAPTCHA_PUBLIC_KEY = settings.RECAPTCHA_PUBLIC_KEY
    else:
        RECAPTCHA_PUBLIC_KEY = ''

    # load template
    t = loader.get_template('NearBeach/authentication/login.html')

    # Get random number
    cryptogen = SystemRandom()

    # context
    c = {
        'LoginForm': form,
        'RECAPTCHA_PUBLIC_KEY': RECAPTCHA_PUBLIC_KEY,
        'image_number': '%(number)03d' % {'number': 1 + cryptogen.randrange(1,19)},
    }

    return HttpResponse(t.render(c, request))


def logout(request):
    # log the user out and go to login page
    auth.logout(request)
    return HttpResponseRedirect(reverse('login'))


@login_required(login_url='login',redirect_field_name="")
def permission_denied(request):
    # Load the template
    t = loader.get_template('NearBeach/authentication/permission_denied.html')

    # context
    c = {
    }

    return HttpResponse(t.render(c, request))



from django.urls import reverse


def assertEqual200(response_array, self):
    # Look through the array to see if it response with 405
    for response in response_array:
        self.assertEqual(
            response.status_code,
            200
        )


def assertEqual405(response_array, self):
    # Look through the array to see if it response with 405
    for response in response_array:
        self.assertEqual(
            response.status_code,
            405
        )


def assertRedirectsToLogin(response_array, self):
    # Loop through the array to see if it redirects to login
    for response in response_array:
        self.assertRedirects(
            response,
            reverse('login'),
            status_code=302,
            target_status_code=200,
            msg_prefix='',
            fetch_redirect_response=True
        )



from django.http import HttpResponse, HttpResponseBadRequest
from django.template import loader
from django.views.decorators.http import require_http_methods
from django.urls import reverse
from django.contrib.auth.decorators import login_required
from django.core import serializers
from django.core.serializers.json import DjangoJSONEncoder

from NearBeach.forms import SearchForm, NewGroupForm
from NearBeach.models import *

import json


@login_required(login_url='login', redirect_field_name='')
@require_http_methods(['POST'])
def check_group_name(request):
    """

    :param request:
    :return:
    """

    # Check user form
    form = SearchForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Check to see if the group name exists
    group_name_results = group.objects.filter(
        is_deleted=False,
        group_name=form.cleaned_data['search'],
    )

    # Send back data
    return HttpResponse(serializers.serialize('json', group_name_results), content_type='application/json')


@login_required(login_url='login',redirect_field_name="")
def group_information(request, group_id):
    """

    :param request:
    :param group_id:
    :return:
    """

    # Get the template
    t = loader.get_template('NearBeach/groups/group_information.html')

    # Get the data we want
    group_results = group.objects.get(group_id=group_id)
    parent_group_results = group.objects.filter(
        is_deleted=False,
    )

    user_list_results = user_group.objects.filter(
        is_deleted=False,
        group_id=group_id,
    ).values(
        'username',
        'username__first_name',
        'username__last_name',
        'username__email',
        'group',
        'group__group_name',
        'permission_set',
        'permission_set__permission_set_name',
    ).order_by(
        'username__first_name',
        'username__last_name',
        'permission_set__permission_set_name',
    )

    # Convert into json
    user_list_results = json.dumps(list(user_list_results), cls=DjangoJSONEncoder)

    # Context
    c = {
        'group_id': group_id,
        'group_results': serializers.serialize('json', [group_results]),
        'parent_group_results': serializers.serialize('json', parent_group_results),
        'user_list_results': user_list_results,
    }

    return HttpResponse(t.render(c, request))


@require_http_methods(['POST'])
@login_required(login_url='login', redirect_field_name='')
def group_information_save(request, group_id):
    """

    :param request:
    :param group_id:
    :return:
    """

    # Check user permissions

    # Get Form Data
    form = NewGroupForm(request.POST)
    if not form.is_valid():
        print(form.errors)
        return HttpResponseBadRequest(form.errors)

    # Update the group's data
    group_update = group.objects.get(group_id=group_id)
    group_update.group_name = form.cleaned_data['group_name']
    group_update.parent_group = form.cleaned_data['parent_group']

    group_update.save()

    return HttpResponse("")


@login_required(login_url='login',redirect_field_name="")
def new_group(request):
    """

    :param request:
    :return:
    """

    # CHeck user permissions

    # Get the template
    t = loader.get_template('NearBeach/groups/new_group.html')

    # Get group data
    group_results = group.objects.filter(
        is_deleted=False,
    ).exclude(
        group_name__in=['Administration'],
    )

    # Get the context
    c = {
        'group_results': serializers.serialize('json', group_results),
    }

    # Return
    return HttpResponse(t.render(c, request))


@require_http_methods(['POST'])
@login_required(login_url='login', redirect_field_name='')
def new_group_save(request):
    """

    :param request:
    :return:
    """

    # Check user permissions

    # Get form data
    form = NewGroupForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Create the new group
    group_submit = group(
        group_name=form.cleaned_data['group_name'],
        parent_group=form.cleaned_data['parent_group'],
        change_user=request.user,
    )
    group_submit.save()

    # Send back the URL for the group
    return HttpResponse(reverse('group_information',args={group_submit.group_id}))



from django.http import HttpResponse, HttpResponseBadRequest
from django.shortcuts import reverse
from django.template import loader
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_http_methods
from django.core import serializers
from django.core.serializers.json import DjangoJSONEncoder
from django.contrib.auth.models import User

from NearBeach.models import *
from NearBeach.forms import *

import json

@login_required(login_url='login',redirect_field_name="")
def new_user(request):
    """

    :param request:
    :return:
    """

    # Add in user permissions

    # Get template
    t = loader.get_template('NearBeach/users/new_user.html')

    # Get context
    c = {}

    return HttpResponse(t.render(c, request))


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def new_user_save(request):
    """

    :param request:
    :return:
    """

    # CHECK USER PERMISSSIONS

    # Get form data
    form = NewUserForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Create the new user
    submit_user = User(
        username=form.cleaned_data['username'],
        first_name=form.cleaned_data['first_name'],
        last_name=form.cleaned_data['last_name'],
        is_active=True,
    )

    # Set the user password
    submit_user.set_password(form.cleaned_data['password1'])

    # Save
    submit_user.save()

    return HttpResponse(reverse('user_information', args={submit_user.id}))

@login_required(login_url='login',redirect_field_name="")
def user_information(request, username):
    """

    :param request:
    :param permission_set_id:
    :return:
    """

    # check user permissions

    # Import template
    t = loader.get_template('NearBeach/users/user_information.html')

    # Get user data
    user_results = User.objects.get(id=username)

    user_list_results = user_group.objects.filter(
        is_deleted=False,
        username=username,
    ).values(
        'username',
        'username__first_name',
        'username__last_name',
        'username__email',
        'group',
        'group__group_name',
        'permission_set',
        'permission_set__permission_set_name',
    ).order_by(
        'username__first_name',
        'username__last_name',
        'permission_set__permission_set_name',
    )
    user_list_results = json.dumps(list(user_list_results), cls=DjangoJSONEncoder)

    # Create the context
    c = {
        'user_list_results': user_list_results,
        'user_results': serializers.serialize('json', [user_results]),
        'username': username,
    }

    return HttpResponse(t.render(c, request))


@require_http_methods(['POST'])
@login_required(login_url='login',redirect_field_name="")
def user_information_save(request, username):
    """

    :param request:
    :param username:
    :return:
    """

    # Add in user permissions

    # Get form
    form = UpdateUserForm(request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest(form.errors)

    # Get user
    update_user = User.objects.get(id=username)

    # Update the user
    update_user.first_name = form.cleaned_data['first_name']
    update_user.last_name = form.cleaned_data['last_name']
    update_user.email = form.cleaned_data['email']
    update_user.is_active = form.cleaned_data['is_active']
    update_user.is_superuser = form.cleaned_data['is_superuser']

    # Save
    update_user.save()

    # Send back blank 200
    return HttpResponse("")